<?php

declare(strict_types=1);

namespace Drupal\notification_message\Cache\Context;

use Drupal\Core\Http\RequestStack;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\ContentEntityType;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Cache\Context\CacheContextInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Define the route entity UUID cache context.
 */
class RouteEntityUuidCacheContext implements CacheContextInterface {

  /**
   * Current request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request|null
   */
  protected $request;

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Define class constructor.
   *
   * @param \Drupal\Core\Http\RequestStack $request_stack
   *   The request stack service.
   */
  public function __construct(
    RequestStack $request_stack,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->request = $request_stack->getCurrentRequest();
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritDoc}
   */
  public static function getLabel(): TranslatableMarkup {
    return new TranslatableMarkup('Route Entity ID');
  }

  /**
   * {@inheritDoc}
   */
  public function getContext(): string {
    $entities = $this->entitiesFromRoute();
    return !empty($entities) ? implode('|', $entities) : 'none';
  }

  /**
   * {@inheritDoc}
   */
  public function getCacheableMetadata(): CacheableMetadata {
    return new CacheableMetadata();
  }

  /**
   * Retrieve the entities from the current route.
   *
   * @return array
   *   An array of entities.
   */
  protected function entitiesFromRoute(): array {
    $entities = [];

    foreach ($this->entityTypeManager->getDefinitions() as $definition) {
      $entity_type = $definition->id();
      if (
        !$definition instanceof ContentEntityType
        || !$this->request->attributes->has($entity_type)
      ) {
        continue;
      }
      /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
      $entity = $this->request->get($entity_type);
      $entities[] = "$entity_type.{$entity->bundle()}.{$entity->uuid()}";
    }

    return $entities;
  }
}
